/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Text Trix code.
 *
 * The Initial Developer of the Original Code is
 * Text Flex.
 * Portions created by the Initial Developer are Copyright (C) 2003-4
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): David Young <dvd@textflex.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

package com.textflex.texttrix;

import javax.swing.*;
import java.io.*;

/** Plugin to replaces both HTML tags and character entities
    with their appropriate plain-text 
    substitutes.  Substitutes include "*" for &lt;b&gt; or &lt;/b&gt;
    for bold tags, "/" for &lt;i&lt; and &lt;/i&lt;, and "--" for 
    "&amp;#151;".  Unrecognized tags are simply removed.
*/
public class HTMLReplacer extends PlugIn { //implements PlugIn {

	/** Constructs the HTML Replacer with descriptive text and 
	images.
	*/
	public HTMLReplacer() {
		super(
			"HTML Replacer",
			"tools",
			"Replaces HTMl with plain-text representations",
			"desc.html",
			"icon.png",
			"icon-roll.png");
	}

	/** Gets the normal icon.
	@return normal icon
	*/
	public ImageIcon getIcon() {
		return getIcon(getIconPath());
	}

	/** Gets the rollover icon.
	@return rollover icon
	*/
	public ImageIcon getRollIcon() {
		return getRollIcon(getRollIconPath());
	}

	/** Gets the detailed, HTML-formatted description.
	For display as a tool tip.
	@return a buffered reader for the description file
	*/
	public BufferedReader getDetailedDescription() {
		return super.getDetailedDescription(getDetailedDescriptionPath());
	}
	
	public PlugInOutcome run(String text) {
		String lowerCase = text.toLowerCase(); // so ignore tag case
		int len = text.length();
		StringBuffer s = new StringBuffer(len);
		int n = 0;//start;
		int end = len;
		int orderedListIndex = -1;
		char c = '\0';

		// append text preceding the selection
		// Add to the stringbuffer char by char, deleting and replacing 
		// tags as appropriate
		while (n < end) {
			// eliminate each line's leading spaces
			while (n < end && ((c = text.charAt(n)) == ' ' || c == '\t'))
				n++;
			while (n < end && (c = text.charAt(n)) != '\n') {
				// tags
				if (c == '<' && n < len - 1) {
					n++;
					// <p> tags
					if (startsWithAny(lowerCase,
						new String[] { "p>", "p " },
						n)) {
						s.append("\n\n");
						// <br> tags
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "br>", "br>" },
							n)) {
						s.append("\n");
						// <b> tags: replace with "*"
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "b>", "b ", "/b>", "/b " },
							n)) {
						s.append("*");
						// <i> tags: replace with "/"
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "i>", "i ", "/i>", "/i " },
							n)) {
						s.append("/");
						// <u> tags: replace with "_"
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "u>", "u ", "/u>", "/u >" },
							n)) {
						s.append("_");
						// <ul> tages: set the list index to -1 for "false"
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "ul>", "ul " },
							n)) {
						orderedListIndex = -1;
						// <ol> tags: reset the list index to 1
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "ol>", "ol " },
							n)) {
						orderedListIndex = 1;
						// <li> tags: add "-" for unordered lists, 
						// the current index for ordered lists
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "li>", "li " },
							n)) {
						if (orderedListIndex == -1) {
							s.append("\n\t-");
						} else {
							s.append("\n\t" + orderedListIndex++ +". ");
						}
						// any other closing tag
					} else if (
						startsWithAny(lowerCase, new String[] { "/" }, n)) {
					}
					int nToBe = -1;
					n =
						((nToBe = lowerCase.indexOf('>', n)) != -1)
							? nToBe
							: end - 1;
					// unicode characters
				} else if (c == '&' && n < len - 1) {
					n++;
					if (startsWithAny(lowerCase,
						new String[] { "nbsp;", "#161;" },
						n)) {
						s.append(" ");
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "#151;" },
							n)) {
						s.append("--");
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "copy;", "#169;" },
							n)) {
						s.append("(c)");
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "reg;", "#174;" },
							n)) {
						s.append("(R)");
					} else if (
						startsWithAny(
							lowerCase,
							new String[] { "#153;" },
							n)) {
						s.append("TM");
					}
					int nToBe = -1;
					// end - 1 so that n won't ever exceed end; n++ later
					n =
						((nToBe = lowerCase.indexOf(';', n)) != -1)
							? nToBe
							: end - 1;
					// skip over hard returns and tabs
				} else if (c == '\n' || c == '\t') {
					// add non-tag, non hard return/tab chars
				} else {
					s.append(c);
				}
				n++;
			}
			// ensures that end on next character
			if (n < end)
				n++;
		}

		// wipe out multiple spaces within the tagless text
		String taglessText = s.toString();
		int taglessLen = taglessText.length();
		s = new StringBuffer(taglessLen);
		for (int i = 0; i < taglessLen; i++) {
			s.append(c = taglessText.charAt(i));
			if (c == ' ') {
				while (i < taglessLen - 1
					&& (c = taglessText.charAt(i + 1)) == ' ')
					i++;
			}
		}
		// stay within original text length;
		// n shouldn't have exceeded end or len, though
		/*
		String finalText = text.substring(0, start) + s.toString();
		if (n < len)
			finalText += text.substring(n);
			*/
		String finalText = s.toString();
		return new PlugInOutcome(finalText);
	}

	/**Checks whether any of the strings in an array are at the start
	 * of another string.
	 * @param s string to check
	 * @param strs[] array of strings that may be at the beginning 
	 * of <code>s</code>
	 * @param offset index to start checking in <code>s</code>
	 * @return <code>true</code> if any of the array's strings 
	 * start <code>s</code>, 
	 * <code>false</code> if otherwise
	 */
	public boolean startsWithAny(String s, String strs[], int offset) {
		for (int i = 0; i < strs.length; i++) {
			//			System.out.println("starts' i: " + i);
			if (s.startsWith(strs[i], offset)) {
				return true;
			}
		}
		return false;
	}

}
