/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Text Trix code.
 *
 * The Initial Developer of the Original Code is
 * Text Flex.
 * Portions created by the Initial Developer are Copyright (C) 2003-4
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): David Young <dvd@textflex.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

package com.textflex.texttrix;

import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

/** Letter Pulse highlights the letters sequentially, in pulses progressing through the 
 * text to spell out a hidden message.  The user sets the message in an options window,
 * which also gives an option for the pulse interval.
 * 
 * <p>The recommended minimum interval is 1 second, for some systems may
 * skip the first letter otherwise during start-up.
 * @author davit
 */
public class LetterPulse extends PlugInWindow {

	private LetterPulseDialog diag = null; // options window
	private String inputstr = ""; // message
	private int inputstrIndex = 0; // position within the message
	private Highlighter highlighter = null; // the threaded pulser

	/** Constructs the Letter Pulse plug-in.
	 * Sets <code>alwaysEntireText</code> to <code>true</code> and creates
	 * the <code>Action</code> to run the plug-in.
	 *
	 */
	public LetterPulse() {
		super(
			"Letter Pulse",
			"trix",
			"Highlight a message by pulsing letters",
			"desc.html",
			"icon.png",
			"icon-roll.png");
		setAlwaysEntireText(true);
		highlighter = new Highlighter();
		

		// Runs the pulser if the user hits the "Pulse" button;
		// creates a shortcut key (alt-R) as an alternative way to invoke
		// the button;
		// pressing the button in the middle of a highlight session simply
		// terminates the highlighter, loads the message, resets its index, 
		// and restarts the highlighter
		Action pulseAction = new AbstractAction("Pulse", null) {
			public void actionPerformed(ActionEvent e) {
				stopHighlighter();
				// creates a new highlighter, such as during the plug-in's first run
				
				inputstr = diag.getInput().toLowerCase();
				inputstrIndex = 0;
				highlighter.start();
			}
		};
		LibTTx.setAcceleratedAction(
			pulseAction,
			"Pulse",
			'P',
			KeyStroke.getKeyStroke("alt P"));
		
		// Stop Action (alt S): creates the action for the button to 
		// prematurely terminate the highlighter
		Action stopPulseAction = new AbstractAction("Stop", null) {
			public void actionPerformed(ActionEvent e) {
				stopHighlighter();
			}
		};
		LibTTx.setAcceleratedAction(
			stopPulseAction,
			"Stop",
			'p',
			KeyStroke.getKeyStroke("alt S"));
			
		// creates the Letter Pulse control window
		diag = new LetterPulseDialog(pulseAction, stopPulseAction);
		/*
		diag.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				stopHighlighter();
			}
		});
		*/
		setWindow(diag);
		//setPanel(diag)
	}
	
	public void stopPlugIn() {
		stopHighlighter();
	}
	
	/**Stops the highlighter.
	 * Useful to stop it both prematurely and at the natural end of the highlight.
	 *
	 */
	public void stopHighlighter() {
		if (highlighter != null) {
			// stop the current highlighter in case it's running
			highlighter.requestStop();
		}
	}

	/** Gets the normal icon.
	@return normal icon
	*/
	public ImageIcon getIcon() {
		ImageIcon pic = getIcon(getIconPath());
		diag.setIconImage(pic);
		return pic;
	}

	/** Gets the rollover icon.
	@return rollover icon
	*/
	public ImageIcon getRollIcon() {
		return getRollIcon(getRollIconPath());
	}

	/** Gets the detailed, HTML-formatted description.
	For display as a tool tip.
	@return a buffered reader for the description file
	*/
	public BufferedReader getDetailedDescription() {
		return super.getDetailedDescription(getDetailedDescriptionPath());
	}
	
	/**Runs the plug-in, starting at the first position in the text.
	 * 
	 */
	public PlugInOutcome run(String s) {
		return run(s, 0, 0);
	}

	/** Runs the plug-in.
	 * Assumes that the calling function has already retrieved the message
	 * and converted it to lower case.  Looks for a single letter at the current
	 * position in the message and increments that position.  Also assumes
	 * that the text's highlight position has not moved.  Otherwise, the plug-in
	 * may not highlight progressively through the text, though still sequentially
	 * through the message.  If no more positions in the message remain, the 
	 * function returns.   The moment that a letter cannot be found, the caret
	 * is set to the end of the text so that no further runs find letters; otherwise,
	 * letters would be skipped.
	 * @param s text to search
	 * @param x start position
	 * @param y end position, non-inclusive
	 * @return the original text and positions to highlight
	*/
	public PlugInOutcome run(String s, int x, int y) {
		String lowerstr = s.toLowerCase();
		String newstr = "";
		if (inputstrIndex == 0)
			y = 0;
		if (inputstrIndex < inputstr.length()) {
			int loc = lowerstr.indexOf(inputstr.charAt(inputstrIndex), y);
			if (loc != -1) {
				// sets noTextChange to "true" to prevent changing the text and flagging it as such
				return new PlugInOutcome(s, loc, loc + 1, true);
			} else if (y < s.length()) {
				// sets the caret to the end of the text to quickly terminate future runs 
				// sets noTextChange to "true" to prevent changing the text and flagging it as such
				return new PlugInOutcome(s, y + 1, y + 1, true);
			}
		}
		return new PlugInOutcome(s);
	}

	/** Thread class to sequentially highlight the message letters with
	 * the set interval between each highlight.
	 * @author davit
	 */
	private class Highlighter extends StoppableThread {
		private Thread thread = null; // the thread itself
		private boolean loop = false; // flags whether to repeat highlighting
		private int speed = 1;

		/** Constructs the pulser.
		 */
		public Highlighter() {
		}

		/** Starts the highlighter so long as <code>thread</code> is null
		 * 
		 */
		public void start() {
			setStopped(false);
			thread = new Thread(this, "Thread");
			thread.start();
		}
		

		/** Runs the highlighter until <code>thread</code> becomes null
		 * or the highlighter has progressed through the whole message.
		 */
		public void run() {
			do {
				while (!interrupted() && inputstrIndex < inputstr.length()) {
					try {
						runPlugIn(); // calls TextTrix to call the run function
						Thread.sleep(getSpeed() * 1000); // interval
						inputstrIndex++; // increment the message position
					} catch (InterruptedException e) {
						setStopped(true);
						Thread.currentThread().interrupt();
					}
				}
				inputstrIndex = 0;
			} while (isLoop() && !isStopped());
			setStopped(true);
		}

		/** Stops the thread.
		 */
		public void requestStop() {
			setStopped(true);
			if (thread != null) {
				thread.interrupt();
				thread = null;
			}
		}
		
		/**Checks if the control panel has its loop box selected.
		 * 
		 */
		public boolean isLoop() {
			// invokeLater to place the command on the main dispatch thread
			// and avoid possible conflicts from multiple threads working
			// on the same object
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					loop = diag.isLoop();
				}
			});
			return loop;
		}
		
		/**Gets the speed in ms from the control panel's speed slider.
		 * 
		 * @return the speed in ms
		 */
		public int getSpeed() {
			// invokeLater to follow the single-thread rule
			EventQueue.invokeLater(new Runnable() {
				public void run() {
					speed = diag.getSpeed();
				}
			});
			return speed;
		}

	}
}

/** The Letter Pulse's options window.
 * 
 * @author davit
 */
class LetterPulseDialog extends JPanel {//JFrame {
	private JLabel inputLbl = null; // label for message
	private JTextField inputFld = null; // area to type message 
	private JLabel speedLbl = null; // label for the interval period
	private JSlider speedSld = null; // slider for the interval
	private JButton pulseBtn = null; // button to invoke the plug-in
	private JButton stopPulseBtn = null; // button to prematurely stop highlighting
	private JCheckBox loopChk = null; // check box to flag loop

	/** Constructs the options window.
	 * 
	 * @param pulseAction runs the plug-in
	 */
	public LetterPulseDialog(Action pulseAction, Action stopPulseAction) {
		//super("Letter Pulse");
		super(new GridBagLayout());
		setSize(400, 150);
		//Container contentPane = getContentPane();
		//contentPane.setLayout(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.anchor = GridBagConstraints.CENTER;
		
		// Creates the loop check box
		String loopChkTxt = "Loop";
		String loopChkTipTxt = 
			"Pete and Repeat went into the store.  Pete came out."
			+ "Who was left inside?";
		loopChk = new JCheckBox(loopChkTxt);
		loopChk.setToolTipText(loopChkTipTxt);

		inputLbl = new JLabel("Letters to locate:");
		LibTTx.addGridBagComponent(
			inputLbl,
			constraints,
			0,
			0,
			1,
			1,
			100,
			0,
			this);//contentPane);

		inputFld = new JTextField(20);
		LibTTx.addGridBagComponent(
			inputFld,
			constraints,
			1,
			0,
			2,
			1,
			100,
			0,
			this);//contentPane);

		speedLbl = new JLabel("Secs between pulses:");
		LibTTx.addGridBagComponent(
			speedLbl,
			constraints,
			0,
			1,
			1,
			1,
			100,
			0,
			this);//contentPane);

		// sets 1 second tick marks
		speedSld = new JSlider(JSlider.HORIZONTAL, 0, 15, 2);
		speedSld.setMajorTickSpacing(5);
		speedSld.setMinorTickSpacing(1);
		speedSld.setPaintTicks(true);
		speedSld.setPaintLabels(true);
		LibTTx.addGridBagComponent(
			speedSld,
			constraints,
			1,
			1,
			3,
			1,
			100,
			0,
			this);//contentPane);

		pulseBtn = new JButton(pulseAction);
		LibTTx.addGridBagComponent(
			pulseBtn,
			constraints,
			0,
			2,
			1,
			1,
			100,
			0,
			this);//contentPane);
		stopPulseBtn = new JButton(stopPulseAction);
		LibTTx.addGridBagComponent(
			stopPulseBtn,
			constraints,
			1,
			2,
			1,
			1,
			100,
			0,
			this);//contentPane);
		
		LibTTx.addGridBagComponent(
			loopChk,
			constraints,
			2,
			2,
			1,
			1,
			0,
			0,
			this);//contentPane);
	}

	/** Sets the window's frame icon.
	 * 
	 * @param pic icon
	 */
	public void setIconImage(ImageIcon pic) {
		//setIconImage(pic.getImage());
	}

	/** Gets the interval time period, in seconds.
	 * 
	 * @return speed value
	 */
	public int getSpeed() {
		return speedSld.getValue();
	}
	/** Gets the input text, the message to highlight.
	 * 
	 * @return the message to highlight
	 */
	public String getInput() {
		return inputFld.getText();
	}
	
	/**Checks if the loop box is selected.
	 * 
	 * @return <code>true</code> if the box has been selected
	 */
	public boolean isLoop() {
		return loopChk.isSelected();
	}

}